@@ -44,15 +44,14 @@ import pandas as pd
 import cv2
 import toml
 from tqdm import tqdm
-from scipy import interpolate
 from collections import Counter
 from anytree import RenderTree
 from anytree.importer import DictImporter
 import logging
 
 from Pose2Sim.common import retrieve_calib_params, computeP, weighted_triangulation, \
-    reprojection, euclidean_distance, sort_people_sports2d, \
-    sort_stringlist_by_last_number, min_with_single_indices, zup2yup, convert_to_c3d
+    reprojection, euclidean_distance, sort_people_sports2d, interpolate_zeros_nans, \
+    sort_stringlist_by_last_number, zup2yup, convert_to_c3d
 from Pose2Sim.skeletons import *
 
 
@@ -61,65 +60,70 @@ __author__ = "David Pagnon"
 __copyright__ = "Copyright 2021, Pose2Sim"
 __credits__ = ["David Pagnon"]
 __license__ = "BSD 3-Clause License"
-__version__ = "0.9.4"
+from importlib.metadata import version
+__version__ = version('pose2sim')
 __maintainer__ = "David Pagnon"
 __email__ = "contact@david-pagnon.com"
 __status__ = "Development"
 
 
 ## FUNCTIONS
-def interpolate_zeros_nans(col, *args):
+def count_persons_in_json(file_path):
     '''
-    Interpolate missing points (of value zero),
-    unless more than N contiguous values are missing.
+    Count the number of persons in a json file.
 
-    INPUTS:
-    - col: pandas column of coordinates
-    - args[0] = N: max number of contiguous bad values, above which they won't be interpolated
-    - args[1] = kind: 'linear', 'slinear', 'quadratic', 'cubic'. Default: 'cubic'
+    INPUT:
+    - file_path: path to the json file
 
     OUTPUT:
-    - col_interp: interpolated pandas column
+    - int: number of persons in the json file
     '''
 
-    if len(args)==2:
-        N, kind = args
-    if len(args)==1:
-        N = np.inf
-        kind = args[0]
-    if not args:
-        N = np.inf
-    
-    # Interpolate nans
-    mask = ~(np.isnan(col) | col.eq(0)) # true where nans or zeros
-    idx_good = np.where(mask)[0]
-    if len(idx_good) <= 4:
-        return col
-        
-    if 'kind' not in locals(): # 'linear', 'slinear', 'quadratic', 'cubic'
-        f_interp = interpolate.interp1d(idx_good, col[idx_good], kind="linear", bounds_error=False)
-    else:
-        f_interp = interpolate.interp1d(idx_good, col[idx_good], kind=kind, fill_value='extrapolate', bounds_error=False)
-    col_interp = np.where(mask, col, f_interp(col.index)) #replace at false index with interpolated values
-    
-    # Reintroduce nans if length of sequence > N
-    idx_notgood = np.where(~mask)[0]
-    gaps = np.where(np.diff(idx_notgood) > 1)[0] + 1 # where the indices of true are not contiguous
-    sequences = np.split(idx_notgood, gaps)
-    if sequences[0].size>0:
-        for seq in sequences:
-            if len(seq) > N: # values to exclude from interpolation are set to false when they are too long 
-                col_interp[seq] = np.nan
-    
-    return col_interp
-
-
-def count_persons_in_json(file_path):
     with open(file_path, 'r') as file:
         data = json.load(file)
         return len(data.get('people', []))
     
 
+def indices_of_first_last_non_nan_chunks(series, min_chunk_size=10):
+    '''
+    Find indices of the first and last chunks of at least min_chunk_size consecutive non-NaN values.
+
+    INPUT:
+    - series: pandas Series to trim
+    - min_chunk_size: minimum size of consecutive non-NaN values to consider (default: 5)
+
+    OUTPUT:
+    - tuple: (start_index, end_index) of the first and last valid chunks
+    '''
+    
+    non_nan_mask = ~np.isnan(series.values)
+    
+    # Find runs of consecutive non-NaN values (eg [(8, 15), (16, 17), (19, 26)])
+    runs = []
+    run_start = None
+    for i, bool_val in enumerate(non_nan_mask):
+        if bool_val and run_start is None:
+            run_start = i
+        elif not bool_val and run_start is not None:
+            run_end = i
+            runs.append((run_start, run_end))
+            run_start = None
+    if run_start is not None:
+        runs.append((run_start, len(non_nan_mask)))
+    
+    # Find runs that have at least min_chunk_size consecutive non-NaN values
+    valid_runs = [(start, end) for start, end in runs if end - start >= min_chunk_size]
+    if not valid_runs:
+        return(0,0)
+    
+    # Get the start of the first valid run and the end of the last valid run
+    first_run_start = valid_runs[0][0] + series.index.start
+    last_run_end = valid_runs[-1][1] + series.index.start
+    
+    # Return the trimmed series
+    return first_run_start, last_run_end
+
+
 def make_trc(config_dict, Q, keypoints_names, f_range, id_person=-1):
     '''
     Make Opensim compatible trc file from a dataframe with 3D coordinates
@@ -156,9 +160,10 @@ def make_trc(config_dict, Q, keypoints_names, f_range, id_person=-1):
                 raise
             frame_rate = round(cap.get(cv2.CAP_PROP_FPS))
         except:
-            frame_rate = 60
+            logging.warning(f'Cannot read video. Frame rate will be set to 60 fps.')
+            frame_rate = 30  
 
-    trc_f = f'{seq_name}_{f_range[0]}-{f_range[1]}.trc'
+    trc_f = f'{seq_name}_{f_range[0]}-{f_range[1]-1}.trc'
 
     #Header
     DataRate = CameraRate = OrigDataRate = frame_rate
@@ -166,7 +171,7 @@ def make_trc(config_dict, Q, keypoints_names, f_range, id_person=-1):
     NumMarkers = len(keypoints_names)
     header_trc = ['PathFileType\t4\t(X/Y/Z)\t' + trc_f, 
             'DataRate\tCameraRate\tNumFrames\tNumMarkers\tUnits\tOrigDataRate\tOrigDataStartFrame\tOrigNumFrames', 
-            '\t'.join(map(str,[DataRate, CameraRate, NumFrames, NumMarkers, 'm', OrigDataRate, f_range[0], f_range[1]])),
+            '\t'.join(map(str,[DataRate, CameraRate, NumFrames, NumMarkers, 'm', OrigDataRate, f_range[0], NumFrames])),
             'Frame#\tTime\t' + '\t\t\t'.join(keypoints_names) + '\t\t\t',
             '\t\t'+'\t'.join([f'X{i+1}\tY{i+1}\tZ{i+1}' for i in range(len(keypoints_names))]) + '\t']
     
@@ -225,7 +230,7 @@ def retrieve_right_trc_order(trc_paths):
     return trc_id
 
 
-def recap_triangulate(config_dict, error, nb_cams_excluded, keypoints_names, cam_excluded_count, interp_frames, non_interp_frames, trc_path):
+def recap_triangulate(config_dict, error, nb_cams_excluded, keypoints_names, cam_excluded_count, interp_frames, non_interp_frames, f_range_trimmed, trc_path):
     '''
     Print a message giving statistics on reprojection errors (in pixel and in m)
     as well as the number of cameras that had to be excluded to reach threshold 
@@ -248,7 +253,8 @@ def recap_triangulate(config_dict, error, nb_cams_excluded, keypoints_names, cam
     # if single trial
     session_dir = session_dir if 'Config.toml' in os.listdir(session_dir) else os.getcwd()
     calib_dir = [os.path.join(session_dir, c) for c in os.listdir(session_dir) if os.path.isdir(os.path.join(session_dir, c)) and  'calib' in c.lower()][0]
-    calib_file = glob.glob(os.path.join(calib_dir, '*.toml'))[0] # lastly created calibration file
+    calib_files = glob.glob(os.path.join(calib_dir, '*.toml'))
+    calib_file = max(calib_files, key=os.path.getctime) # lastly created calibration file
     calib = toml.load(calib_file)
     cal_keys = [c for c in calib.keys() 
             if c not in ['metadata', 'capture_volume', 'charuco', 'checkerboard'] 
@@ -298,7 +304,7 @@ def recap_triangulate(config_dict, error, nb_cams_excluded, keypoints_names, cam
         mean_error_mm = np.around(mean_error_px * Dm / fm *1000, decimals=1)
         mean_cam_excluded = np.around(nb_cams_excluded[n]['mean'].mean(), decimals=2)
 
-        logging.info(f'\n--> Mean reprojection error for all points on all frames is {mean_error_px} px, which roughly corresponds to {mean_error_mm} mm. ')
+        logging.info(f'\n--> Mean reprojection error for all points on frames {f_range_trimmed[n][0]} to {f_range_trimmed[n][1]} is {mean_error_px} px, which roughly corresponds to {mean_error_mm} mm. ')
         logging.info(f'Cameras were excluded if likelihood was below {likelihood_threshold} and if the reprojection error was above {error_threshold_triangulation} px.') 
         if interpolation_kind != 'none':
             logging.info(f'Gaps were interpolated with {interpolation_kind} method if smaller than {interp_gap_smaller_than} frames. Larger gaps were filled with {["the last valid value" if fill_large_gaps_with == "last_value" else "zeros" if fill_large_gaps_with == "zeros" else "NaNs"][0]}.') 
@@ -660,7 +666,8 @@ def triangulate_all(config_dict):
     except:
         raise Exception(f'No .toml calibration direcctory found.')
     try:
-        calib_file = glob.glob(os.path.join(calib_dir, '*.toml'))[0] # lastly created calibration file
+        calib_files = glob.glob(os.path.join(calib_dir, '*.toml'))
+        calib_file = max(calib_files, key=os.path.getctime) # lastly created calibration file
     except:
         raise Exception(f'No .toml calibration file found in the {calib_dir}.')
     pose_dir = os.path.join(project_dir, 'pose')
@@ -698,9 +705,14 @@ def triangulate_all(config_dict):
     #     print(f'{pre}{node.name} id={node.id}')
     
     # left/right swapped keypoints
-    keypoints_names_swapped = ['L'+keypoint_name[1:] if keypoint_name.startswith('R') else 'R'+keypoint_name[1:] if keypoint_name.startswith('L') else keypoint_name for keypoint_name in keypoints_names]
-    keypoints_names_swapped = [keypoint_name_swapped.replace('right', 'left') if keypoint_name_swapped.startswith('right') else keypoint_name_swapped.replace('left', 'right') if keypoint_name_swapped.startswith('left') else keypoint_name_swapped for keypoint_name_swapped in keypoints_names_swapped]
-    keypoints_idx_swapped = [keypoints_names.index(keypoint_name_swapped) for keypoint_name_swapped in keypoints_names_swapped] # find index of new keypoint_name
+    try:
+        keypoints_names_swapped = ['L'+keypoint_name[1:] if keypoint_name.startswith('R') else 'R'+keypoint_name[1:] if keypoint_name.startswith('L') else keypoint_name for keypoint_name in keypoints_names]
+        keypoints_names_swapped = [keypoint_name_swapped.replace('right', 'left') if keypoint_name_swapped.startswith('right') else keypoint_name_swapped.replace('left', 'right') if keypoint_name_swapped.startswith('left') else keypoint_name_swapped for keypoint_name_swapped in keypoints_names_swapped]
+        keypoints_idx_swapped = [keypoints_names.index(keypoint_name_swapped) for keypoint_name_swapped in keypoints_names_swapped] # find index of new keypoint_name
+    except:
+        keypoints_names_swapped = keypoints_names
+        keypoints_idx_swapped = keypoints_idx
+        logging.warning('No left/right swap was performed.')
     
     # 2d-pose files selection
     try:
@@ -726,7 +738,7 @@ def triangulate_all(config_dict):
     json_files_names = [sort_stringlist_by_last_number(js) for js in json_files_names]    
 
     # frame range selection
-    f_range = [[0,min([len(j) for j in json_files_names])] if frame_range==[] else frame_range][0]
+    f_range = [[0,min([len(j) for j in json_files_names])] if frame_range in ('all', 'auto', []) else frame_range][0]
     frame_nb = f_range[1] - f_range[0]
     
     # Check that camera number is consistent between calibration file and pose folders
@@ -744,7 +756,7 @@ def triangulate_all(config_dict):
     error = [[] for n in range(nb_persons_to_detect)]
     nb_cams_excluded = [[] for n in range(nb_persons_to_detect)]
     id_excluded_cams = [[] for n in range(nb_persons_to_detect)]
-    Q_tot, error_tot, nb_cams_excluded_tot,id_excluded_cams_tot = [], [], [], []
+    Q_tot, error_tot, nb_cams_excluded_tot, id_excluded_cams_tot, f_range_trimmed = [], [], [], [], []
     for f in tqdm(range(*f_range)):
         # print(f'\nFrame {f}:')        
         # Get x,y,likelihood values from files
@@ -835,54 +847,52 @@ def triangulate_all(config_dict):
     id_excluded_cams_tot = [list(tpl) for tpl in zip(*it.zip_longest(*id_excluded_cams_tot, fillvalue=[np.nan]*keypoints_nb*3))]
 
     # dataframes for each person
-    Q_tot = [pd.DataFrame([Q_tot_f[n] for Q_tot_f in Q_tot]) for n in range(nb_persons_to_detect)]
-    error_tot = [pd.DataFrame([error_tot_f[n] for error_tot_f in error_tot]) for n in range(nb_persons_to_detect)]
-    nb_cams_excluded_tot = [pd.DataFrame([nb_cams_excluded_tot_f[n] for nb_cams_excluded_tot_f in nb_cams_excluded_tot]) for n in range(nb_persons_to_detect)]
-    id_excluded_cams_tot = [pd.DataFrame([id_excluded_cams_tot_f[n] for id_excluded_cams_tot_f in id_excluded_cams_tot]) for n in range(nb_persons_to_detect)]
-    
-    for n in range(nb_persons_to_detect):
-        error_tot[n]['mean'] = error_tot[n].mean(axis = 1)
-        nb_cams_excluded_tot[n]['mean'] = nb_cams_excluded_tot[n].mean(axis = 1)
-    
-    # Delete participants with less than 4 valid triangulated frames
-    # for each person, for each keypoint, frames to interpolate
-    zero_nan_frames = [np.where( Q_tot[n].iloc[:,::3].T.eq(0) | ~np.isfinite(Q_tot[n].iloc[:,::3].T) ) for n in range(nb_persons_to_detect)]
-    zero_nan_frames_per_kpt = [[zero_nan_frames[n][1][np.where(zero_nan_frames[n][0]==k)[0]] for k in range(keypoints_nb)] for n in range(nb_persons_to_detect)]
-    non_nan_nb_first_kpt = [frame_nb - len(zero_nan_frames_per_kpt[n][0]) for n in range(nb_persons_to_detect)]
-    deleted_person_id = [n for n in range(len(non_nan_nb_first_kpt)) if non_nan_nb_first_kpt[n]<4]
+    Q_tot = [pd.DataFrame([Q_tot_f[n] for Q_tot_f in Q_tot], index=range(*f_range)) for n in range(nb_persons_to_detect)]
+    error_tot = [pd.DataFrame([error_tot_f[n] for error_tot_f in error_tot], index=range(*f_range)) for n in range(nb_persons_to_detect)]
+    nb_cams_excluded_tot = [pd.DataFrame([nb_cams_excluded_tot_f[n] for nb_cams_excluded_tot_f in nb_cams_excluded_tot], index=range(*f_range)) for n in range(nb_persons_to_detect)]
+    id_excluded_cams_tot = [pd.DataFrame([id_excluded_cams_tot_f[n] for id_excluded_cams_tot_f in id_excluded_cams_tot], index=range(*f_range)) for n in range(nb_persons_to_detect)]
 
+    for n in range(nb_persons_to_detect):
+        error_tot[n]['mean'] = error_tot[n].mean(axis=1,skipna=False)
+        nb_cams_excluded_tot[n]['mean'] = nb_cams_excluded_tot[n].mean(axis=1)
+    
+    # import pickle
+    # with open(os.path.join(session_dir, 'all.pkl'), 'wb') as f:
+    #     pickle.dump([Q_tot, error_tot, nb_cams_excluded_tot, id_excluded_cams_tot, zero_nan_frames_per_kpt], f)
+    ## with open(os.path.join(session_dir, 'all.pkl'), 'rb') as f:
+    ##     Q_tot, error_tot, nb_cams_excluded_tot, id_excluded_cams_tot, zero_nan_frames_per_kpt = pickle.load(f)
+    # Q_tot[0].to_csv(os.path.join(session_dir, 'Q_tot.csv'), index=False, sep='\t')
+    # error_tot[0].to_csv(os.path.join(session_dir, 'error_tot.csv'), index=False, sep='\t')
+
+    # Trim around good frames and remove persons with too few frames
+    f_range_trimmed = [indices_of_first_last_non_nan_chunks(err['mean'], interp_gap_smaller_than) for err in error_tot]
+    # f_range_trimmed = [f_range]*nb_persons_to_detect
+    deleted_person_id = [n for n, f_range in enumerate(f_range_trimmed) if len(range(*f_range))<4]
     Q_tot = [Q_tot[n] for n in range(len(Q_tot)) if n not in deleted_person_id]
     error_tot = [error_tot[n] for n in range(len(error_tot)) if n not in deleted_person_id]
     nb_cams_excluded_tot = [nb_cams_excluded_tot[n] for n in range(len(nb_cams_excluded_tot)) if n not in deleted_person_id]
     id_excluded_cams_tot = [id_excluded_cams_tot[n] for n in range(len(id_excluded_cams_tot)) if n not in deleted_person_id]
     nb_persons_to_detect = len(Q_tot)
-
-    if nb_persons_to_detect ==0:
+    if nb_persons_to_detect == 0:
         raise Exception('No persons have been triangulated. Please check your calibration and your synchronization, or the triangulation parameters in Config.toml.')
 
-    # IDs of excluded cameras
-    # id_excluded_cams_tot = [np.concatenate([id_excluded_cams_tot[f][k] for f in range(frames_nb)]) for k in range(keypoints_nb)]
-    id_excluded_cams_tot = [np.hstack(np.hstack(np.array(id_excluded_cams_tot[n]))) for n in range(nb_persons_to_detect)]
-    cam_excluded_count = [dict(Counter(k)) for k in id_excluded_cams_tot]
-    [cam_excluded_count[n].update((x, y/frame_nb/keypoints_nb) for x, y in cam_excluded_count[n].items()) for n in range(nb_persons_to_detect)]
-    
-    # Optionally, for each person, for each keypoint, show indices of frames that should be interpolated
-    if show_interp_indices:
-        gaps = [[np.where(np.diff(zero_nan_frames_per_kpt[n][k]) > 1)[0] + 1 for k in range(keypoints_nb)] for n in range(nb_persons_to_detect)]
-        sequences = [[np.split(zero_nan_frames_per_kpt[n][k], gaps[n][k]) for k in range(keypoints_nb)] for n in range(nb_persons_to_detect)]
-        interp_frames = [[[f'{seq[0]}:{seq[-1]}' for seq in seq_kpt if len(seq)<=interp_gap_smaller_than and len(seq)>0] for seq_kpt in sequences[n]] for n in range(nb_persons_to_detect)]
-        non_interp_frames = [[[f'{seq[0]}:{seq[-1]}' for seq in seq_kpt if len(seq)>interp_gap_smaller_than] for seq_kpt in sequences[n]] for n in range(nb_persons_to_detect)]
-    else:
-        interp_frames = None
-        non_interp_frames = []
+    logging.info("Trimming " + ", ".join([f"Participant {n} around frames {f_range_trimmed[n]}" for n in range(nb_persons_to_detect)])+".")
+    Q_tot = [Q_tot[n].loc[f_range_trimmed[n][0]:f_range_trimmed[n][1]-1] for n in range(nb_persons_to_detect)]
+    error_tot = [error_tot[n].loc[f_range_trimmed[n][0]:f_range_trimmed[n][1]-1] for n in range(nb_persons_to_detect)]
+    nb_cams_excluded_tot = [nb_cams_excluded_tot[n].loc[f_range_trimmed[n][0]:f_range_trimmed[n][1]-1] for n in range(nb_persons_to_detect)]
+    id_excluded_cams_tot = [id_excluded_cams_tot[n].loc[f_range_trimmed[n][0]:f_range_trimmed[n][1]-1] for n in range(nb_persons_to_detect)]
+    zero_nan_frames = [np.where( Q_tot[n].loc[:,::3].T.eq(0) | ~np.isfinite(Q_tot[n].iloc[:,::3].T) ) for n in range(nb_persons_to_detect)]
+    zero_nan_frames_per_kpt = [[zero_nan_frames[n][1][np.where(zero_nan_frames[n][0]==k)[0]] for k in range(keypoints_nb)] for n in range(nb_persons_to_detect)]
+    zero_nan_frames_per_kpt = [[z[(f_range_trimmed[n][0] < z) & (f_range_trimmed[n][1] > z)] for z in zero_nan_frames_per_kpt[n]] for n in range(nb_persons_to_detect)]
 
     # Interpolate missing values
     if interpolation_kind != 'none':
         for n in range(nb_persons_to_detect):
             try:
-                Q_tot[n] = Q_tot[n].apply(interpolate_zeros_nans, axis=0, args = [interp_gap_smaller_than, interpolation_kind])
+                Q_tot[n] = Q_tot[n].apply(interpolate_zeros_nans, axis=0, args=[interp_gap_smaller_than, interpolation_kind])
             except:
                 logging.info(f'Interpolation was not possible for person {n}. This means that not enough points are available, which is often due to a bad calibration.')
+
     # Fill non-interpolated values with last valid one
     if fill_large_gaps_with == 'last_value':
         for n in range(nb_persons_to_detect): 
@@ -890,9 +900,9 @@ def triangulate_all(config_dict):
     elif fill_large_gaps_with == 'zeros':
         for n in range(nb_persons_to_detect): 
             Q_tot[n].replace(np.nan, 0, inplace=True)
-    
+
     # Create TRC file
-    trc_paths = [make_trc(config_dict, Q_tot[n], keypoints_names, f_range, id_person=n) for n in range(len(Q_tot))]
+    trc_paths = [make_trc(config_dict, Q_tot[n], keypoints_names, f_range_trimmed[n], id_person=n) for n in range(len(Q_tot))]
     if make_c3d:
         c3d_paths = [convert_to_c3d(t) for t in trc_paths]
         
@@ -909,9 +919,23 @@ def triangulate_all(config_dict):
     #     cam_excluded_count = [cam_excluded_count[i] for i in trc_id]
     #     interp_frames = [interp_frames[i] for i in trc_id]
     #     non_interp_frames = [non_interp_frames[i] for i in trc_id]
-        
     #     logging.info('\nThe trc and c3d files have been renamed to match the order of the static sequences.')
 
+    # IDs of excluded cameras
+    # id_excluded_cams_tot = [np.concatenate([id_excluded_cams_tot[f][k] for f in range(frames_nb)]) for k in range(keypoints_nb)]
+    id_excluded_cams_tot = [np.hstack(np.hstack(np.array(id_excluded_cams_tot[n]))) for n in range(nb_persons_to_detect)]
+    cam_excluded_count = [dict(Counter(k)) for k in id_excluded_cams_tot]
+    [cam_excluded_count[n].update((x, y/frame_nb/keypoints_nb) for x, y in cam_excluded_count[n].items()) for n in range(nb_persons_to_detect)]
+
+    # Optionally, for each person, for each keypoint, show indices of frames that should be interpolated
+    if show_interp_indices:
+        gaps = [[np.where(np.diff(zero_nan_frames_per_kpt[n][k]) > 1)[0] + 1 for k in range(keypoints_nb)] for n in range(nb_persons_to_detect)]
+        sequences = [[np.split(zero_nan_frames_per_kpt[n][k], gaps[n][k]) for k in range(keypoints_nb)] for n in range(nb_persons_to_detect)]
+        interp_frames = [[[f'{seq[0]}:{seq[-1]}' for seq in seq_kpt if len(seq)<=interp_gap_smaller_than and len(seq)>0] for seq_kpt in sequences[n]] for n in range(nb_persons_to_detect)]
+        non_interp_frames = [[[f'{seq[0]}:{seq[-1]}' for seq in seq_kpt if len(seq)>interp_gap_smaller_than] for seq_kpt in sequences[n]] for n in range(nb_persons_to_detect)]
+    else:
+        interp_frames = None
+        non_interp_frames = []
 
     # Recap message
-    recap_triangulate(config_dict, error_tot, nb_cams_excluded_tot, keypoints_names, cam_excluded_count, interp_frames, non_interp_frames, trc_paths)
+    recap_triangulate(config_dict, error_tot, nb_cams_excluded_tot, keypoints_names, cam_excluded_count, interp_frames, non_interp_frames, f_range_trimmed, trc_paths)
